/*
 * array.h
 *
 *  Created on: 03.08.2009
 *      Author: afestini
 *
 * Helper functions for fixed size C arrays
 *
 * Mostly the same as calling std functions with (array, array+size), except
 * automatically deriving array size (array must be static, not a pointer)
 */

#ifndef ARRAY_H_
#define ARRAY_H_

/*
 * Get number of elements in array
 */
template<typename T, size_t N>
inline size_t size( T const (&)[N] )
{
    return N;
}

/*
 * Return position of (first occurrence of) element
 */
template<typename T, size_t N>
inline size_t find( T const (&a)[N], const T& elem )
{
    return std::find(a, a+N, elem) - a;
}

template<typename T, size_t N, class CMP>
inline size_t findIf( T const (&a)[N], const T& elem, CMP cmp )
{
    return std::find_if(a, a+N, elem, cmp) - a;
}

template<typename T, size_t N>
inline bool contains( T const (&a)[N], const T& elem )
{
    return find(a, elem) < N;
}

template<typename T, size_t N>
inline size_t minElement( T const (&a)[N] )
{
    return std::min_element(a, a+N) - a;
}

template<typename T, size_t N>
inline size_t maxElement( T const (&a)[N] )
{
    return std::max_element(a, a+N) - a;
}

#endif /* ARRAY_H_ */
