/*
 * autoobj.h
 *
 *  Created on: 27.07.2009
 *      Author: afestini
 *
 *  Mostly useless and behaves like std::auto_ptr, except allowing to specify
 *  a deallocation function to be called to release the object.
 *
 *	To avoid redundancy, use std::auto_ptr when requiring "delete". When requiring
 *	delete[], think really hard about why you're not using std::vector instead.
 */

#ifndef AUTOOBJ_H_
#define AUTOOBJ_H_

//Taken from GCC STL implementation
template<typename _Tp, void (*dealloc)(_Tp*)>
class auto_obj
{
private:
    _Tp* _M_ptr;

public:
    /// The pointed-to type.
    typedef _Tp element_type;

    explicit auto_obj(element_type* __p = 0) throw() : _M_ptr(__p) {}

    template<typename _Tp1>
    auto_obj(auto_obj<_Tp1, dealloc>& __a) throw() : _M_ptr(__a.release()) {}

    template<typename _Tp1>
    operator auto_obj<_Tp1, dealloc>() throw()
    { return auto_obj<_Tp1, dealloc>(release()); }

    auto_obj& operator=(auto_obj& __a) throw()
    {
      reset(__a.release());
      return *this;
    }

    template<typename _Tp1>
    auto_obj& operator=(auto_obj<_Tp1, dealloc>& __a) throw()
    {
        reset(__a.release());
        return *this;
    }
    ~auto_obj() { dealloc(_M_ptr); }

    operator bool() { return _M_ptr != 0; }
    element_type& operator*() const throw() { return *_M_ptr; }
    element_type* operator->() const throw() { return _M_ptr; }
    element_type* get() const throw() { return _M_ptr; }

    element_type* release() throw()
    {
        element_type* __tmp = _M_ptr;
        _M_ptr = 0;
        return __tmp;
    }

    template<typename _Tp1>
    void reset(_Tp1* __p = 0) throw()
    {
        if (__p != _M_ptr) dealloc(_M_ptr);
        _M_ptr = __p;
    }
};

#endif /* AUTOOBJ_H_ */
