/*
 * factory.h
 *
 *  Created on: 05.08.2009
 *      Author: afestini
 *
 *  Simple factory to create instances of registered objects by some ID.
 *  Default ID type is unsigned, but can be any kind of object being less-than comparable.
 *
 *  Creator functions can be registered manually -registerCreator(id, function)-, but
 *  for common uses Factory::Register<Class>(id) will suffice and automatically
 *  provide a basic creator function.
 *
 *  Call Factory::create(id) to create an instance.
 */

#ifndef FACTORY_INCLUDED
#define FACTORY_INCLUDED

#include <map>

template<class T, typename ID_Type = unsigned>
class Factory
{
public:
	typedef T*(*Creator)();
	typedef T AbstractType;
	typedef ID_Type IDType;

	typedef std::map<IDType, Creator> CreatorTable;

	static void registerCreator(IDType id, Creator creator) { table()[id] = creator; }

	static T* create(IDType id) {
		typename CreatorTable::iterator it = table().find(id);
		return it != table().end() ? it->second() : 0;
	}

	template<class Class>
	struct Register
	{
		static AbstractType* create() { return new Class; }
		explicit Register(const IDType& id) { Factory::registerCreator(id, create); }
	};

protected:
	static CreatorTable& table() { static CreatorTable _table; return _table; }
};

#endif
