/*
 * flagset.h
 *
 *  Created on: 06.08.2009
 *      Author: afestini
 *
 *  Only exists for more convenient test for all or any bits being set.
 *  Internally limited to unsigned, functions defined to be used with
 *  other Flags or any integer types.
 */

#ifndef FLAGSET_H_
#define FLAGSET_H_

class Flags
{
	unsigned bits_;
public:
	template<typename T>
	explicit Flags(T f) : bits_(f) {}

	Flags operator~() { return Flags(~bits_); }
	Flags& flip() { return bits_^=~0, *this; }

	Flags operator|(Flags f) { return Flags(bits_ | f.bits_); }
	Flags& operator|=(Flags f) { return bits_ |= f.bits_, *this; }
	Flags operator&(Flags f) { return Flags(bits_ & f.bits_); }
	Flags& operator&=(Flags f) { return bits_ &= f.bits_, *this; }

	bool any(Flags f) { return (bits_ & f.bits_) != 0; }
	bool all(Flags f) { return (bits_ & f.bits_) == f.bits_; }

	template<typename T>
	Flags operator|(T bits) { return Flags(bits_ | bits); }

	template<typename T>
	Flags& operator|=(T bits) { return bits_ |= bits, *this; }

	template<typename T>
	Flags operator&(T bits) { return Flags(bits_ & bits); }

	template<typename T>
	Flags& operator&=(T bits) { return bits_ &= bits, *this; }

	bool any(unsigned f) { return (bits_ & f) != 0; }
	bool all(unsigned f) { return (bits_ & f) == f; }

	operator bool() { return bits_ != 0; }
};

#endif /* FLAGSET_H_ */
