/*
 * manager.h
 *
 *  Created on: 16.07.2009
 *      Author: afestini
 */

#ifndef MANAGER_H_
#define MANAGER_H_

#include <list>
#include <map>
#include "sharedptr.h"
#include "lockable.h"

template<class T, typename IDType = unsigned>
class Manager : public Lockable<Manager<T,IDType> >
{
public:
	typedef SharedPtr<T, MultiThreaded> PtrType;
	typedef std::map<IDType, PtrType> ObjectTable;
	typedef std::list< PtrType > List;

	PtrType get(IDType id)
	{
		typename ObjectTable::iterator it = _objects.find(id);
		return (it != _objects.end()) ? it->second : SharedPtr<T, MultiThreaded>(0);
	}

	PtrType add(IDType id, T* object)
	{
		return _objects[id] = PtrType(object);
	}

	void remove(IDType id)
	{
		typename ObjectTable::iterator it = _objects.find(id);
		if ( it != _objects.end() )
			_objects.erase(it);
	}

	void remove(T* object)
	{
		typename ObjectTable::iterator it = _objects.begin();
		for ( ; it != _objects.end(); ++it )
			if ( it->second == object ) break;

		if ( it != _objects.end() )
			_objects.erase(it);
	}

	List getList()
	{
		List lst;
		for (typename ObjectTable::iterator it = _objects.begin(); it != _objects.end(); ++it)
			lst.push_back(it->second);
		return lst;
	}

	void clear() { _objects.clear(); }

private:
	ObjectTable _objects;
};

#endif /* MANAGER_H_ */
