/*
 * Properties can be used to have a consistent interface without manually implementing
 * tons of get/set functions, allowing the use of a property wherever access functions
 * don't need to implement any special behavior.
 *
 * Property<int> value; //declare
 * int x = value(); //get
 * value(10); //set
 */

#ifndef PROPERTY_H_
#define PROPERTY_H_

#include "scopedlock.h"

template<typename T>
class Property
{
protected:
	T value;
public:
	Property(const T& val) : value(val) {}

	inline const T& operator()() const { return value; }
	inline const T& operator()(const T& newVal) { return value=newVal; }
};


#endif
