/*
 * scopedlock.h
 *
 *  Created on: 27.07.2009
 *      Author: afestini
 */

#ifndef SCOPEDLOCK_H_
#define SCOPEDLOCK_H_

#include "lockable.h"

//Use ScopedLock lock(mutexName);
//to automatically unlock mutex at end of scope
class ScopedLock
{
public:
    explicit ScopedLock(pthread_mutex_t& m) : mutex(&m) { pthread_mutex_lock(mutex); }

    template<class T>
	explicit ScopedLock(Lockable<T>& obj) : mutex(&obj.mutex) { pthread_mutex_lock(mutex); }

	~ScopedLock() { unlock(); }

    void unlock()
    {
        if (mutex) pthread_mutex_unlock(mutex);
        mutex = 0;
    }

private:
    pthread_mutex_t *mutex;
};


#endif /* SCOPEDLOCK_H_ */
