/*
 * selector.h
 *
 *  Created on: 24.07.2009
 *      Author: afestini
 *
 *  Wrapper for select.
 *  Usage:
 *  	Selector s;
 *  	s.addFD(filedescriptor);
 *  	s.setTimeout(xxx); //optional, either ms or timeval (-1,-1 being no timeout)
 *  	s.select();
 *    	if ( s.checkFD(filedescriptor) ) read...
 */

#ifndef SELECTOR_H_
#define SELECTOR_H_

#include <errno.h>

class Selector
{
	fd_set fds_;
	int maxFD_;
	timeval timeout_;
	bool useTimeout_;

public:
	Selector() : maxFD_(-1), useTimeout_(false) { FD_ZERO(&fds_);	}

	void addFD( int fd )
	{
		maxFD_ = std::max(maxFD_, fd);
		if ( fd >= 0 ) FD_SET(fd, &fds_);
	}

	void setTimeout( int ms )
	{
		timeval t = {ms/1000, (ms%1000)*1000};
		setTimeout(t);
	}

	void setTimeout( const timeval& timeout )
	{
		timeout_ = timeout;
		if ( timeout.tv_sec >= 0 && timeout.tv_usec >= 0 )
			useTimeout_ = true;
	}

	int select()
	{
		int result = -1;
		while ( result < 0 && maxFD_ >= 0 )
		{
			result = ::select(maxFD_ + 1, &fds_, 0, 0, (useTimeout_ ? &timeout_ : 0) );
			if ( result < 0 && errno != EINTR ) break;
		}
		return result;
	}

	bool checkFD( int fd ) { return fd >= 0 ? FD_ISSET(fd, &fds_) : false; }
};

#endif /* SELECTOR_H_ */
