/*
 * simplecallback.h
 *
 *  Created on: 05.08.2009
 *      Author: afestini
 */

#ifndef SIMPLECALLBACK_H_
#define SIMPLECALLBACK_H_

#include "memfunc.h"
#include "sharedptr.h"

struct NullCallback {};

class CallbackBase : public RefCounted
{
public:
	virtual ~CallbackBase()	{}
	virtual void operator()() = 0;
};

template <typename Func>
class CallbackImpl : public CallbackBase
{
	Func func_;
public:
	CallbackImpl( Func func ) : func_(func) {}
	void operator()() { func_(); }
};

class Callback
{
	SharedPtr<CallbackBase> cb_;
public:
	Callback() {}

	template<class Func>
	explicit Callback(Func callback) : cb_(new CallbackImpl<Func>(callback)) {}

	template<class Func>
	void set(Func func) { cb_ = new CallbackImpl<Func>(func); }

	void set(NullCallback&) { cb_ = 0; }

	void operator()() { if (cb_) (*cb_)(); }
};

#endif /* SIMPLECALLBACK_H_ */
