/*
 * thread.h
 *
 *  Created on: 17.07.2009
 *      Author: afestini
 *
 *  Encapsulating thread creation and accepting function pointers or functors as parameter.
 *  Member functions need to be passed with binder(member function pointer, instance).
 *
 *  The thread object will automatically be cleaned up after the thread function returns
 *
 *  e.g.
 *  pthread_t id = runAsThread( someFunction );
 *  or
 *  runAsThread( binder(&MyClass::function, myInstance) );
 */

#ifndef THREAD_H_
#define THREAD_H_

#include "pthread.h"

template<class Function>
class ThreadObject
{
public:
	ThreadObject(Function func)	: function_(func)	{}

	static void* threadFunc(void* threadObject)
	{
		std::auto_ptr<ThreadObject> t( (ThreadObject*)threadObject );
		t->function_();
		return 0;
	}

private:
	Function function_;
};

template<class T>
pthread_t runAsThread(T func)
{
	pthread_t thread = 0;
	pthread_create(&thread, 0, ThreadObject<T>::threadFunc, new ThreadObject<T>(func) );
	return thread;
}

#endif /* THREAD_H_ */
