/*
 * tree.h
 *
 *  Created on: 27.07.2009
 *      Author: afestini
 */

#ifndef TREE_H_
#define TREE_H_

#include <list>
#include <map>

template<typename Key, typename Val>
struct Tree
{
	struct Path
	{
		Tree* tree;
		list<Key> steps;
		Path& operator+(const Key& x) { steps.push_back(x); return *this; }
		//Path& operator/(const Key& x) { return operator+(x); }
		//Path& operator|(const Key& x) { return operator+(x); }

		Path(Tree* t) : tree(t) {}
		Val* get() { return tree->find(*this); }
		bool set(const Val& v) { return tree->set(*this, v); }
	};

	struct Node
	{
		virtual ~Node() {}
		virtual Val* eval( Path& p ) = 0;
		virtual bool set( Path& p, const Val& v ) = 0;
		virtual void print( int d ) = 0;
	};
	struct Branch : public Node
	{
		typedef map<Key, Node*> Table;
		Table branches;
		~Branch()
		{
			for (typename Table::iterator it = branches.begin(); it != branches.end(); ++it)
				delete it->second;
		}
		Val* eval( Path& p )
		{
			typename Table::iterator it = branches.end();
			if ( !p.steps.empty() )
			{
				it = branches.find(p.steps.front());
				p.steps.pop_front();
			}
			return it != branches.end() ? it->second->eval(p) : 0;
		}

		bool set( Path& p, const Val& v )
		{
			bool result = false;
			if (!p.steps.empty())
			{
				Node*& n = branches[p.steps.front()];
				p.steps.pop_front();
				if (!n)
					n = p.steps.empty() ? static_cast<Node*>(new Leaf) : static_cast<Node*>(new Branch);
				result = n->set(p,v);
			}
			return result;
		}

		void print( int d )
		{
			for (typename Table::iterator it = branches.begin(); it != branches.end(); ++it)
			{
				for ( int i=0; i<d; ++i) cout << "  ";
				cout << it->first << endl;
				it->second->print(d+1);
			}
		}
	};
	struct Leaf : public Node
	{
		Val value;
		Val* eval( Path& p ) { return p.steps.empty() ? &value : 0; }
		bool set( Path& p, const Val& v )
		{
			if ( p.steps.empty() ) value = v;
			return p.steps.empty();
		}
		void print(int d)
		{
			for ( int i=0; i<d; ++i) cout << "  ";
			cout << value << endl;
		}
	};

	Branch rootNode;

	Path root() { return Path(this); }

	Val* find( Path p ) { return rootNode.eval(p); }
	bool set( Path p, const Val& v)	{ return rootNode.set(p, v); }

	void print()
	{
		cout << "Root\n";
		rootNode.print(1);
	}
};

#endif /* TREE_H_ */
