/*
 * types.h
 *
 *  Created on: 19.08.2009
 *      Author: afestini
 *
 *  Typedefs to automatically have the correct size independent of the used platform.
 *  If a type isn't available in the specified size it will be typedef'ed to void.
 *
 *  Pattern: [u]<type><bytes> (e.g. int8, uint32, float64)
 */

#ifndef TYPES_BY_SIZE_H_
#define TYPES_BY_SIZE_H_

#include "typelist.h"
#include "typeselect.h"

template<size_t size, typename list>
struct TypeOfSize
{
	typedef typename SelectT
	< sizeof(typename list::Head)==size,
	  typename list::Head,
	  typename TypeOfSize<size, typename list::Tail>::type
	>::Type type;
};

template<size_t size> struct TypeOfSize<size, TypeList<> > { typedef NullType type; };

typedef TypeList<char, short, int, long, long long> SignedType;
typedef TypeList<unsigned char, unsigned short, unsigned int, unsigned long, unsigned long long> UnsignedType;
typedef TypeList<float, double, long double> FloatType;

typedef TypeOfSize<1, SignedType>::type int8;
typedef TypeOfSize<2, SignedType>::type int16;
typedef TypeOfSize<4, SignedType>::type int32;
typedef TypeOfSize<8, SignedType>::type int64;
typedef TypeOfSize<16, SignedType>::type int128;

typedef TypeOfSize<1, UnsignedType>::type uint8;
typedef TypeOfSize<2, UnsignedType>::type uint16;
typedef TypeOfSize<4, UnsignedType>::type uint32;
typedef TypeOfSize<8, UnsignedType>::type uint64;
typedef TypeOfSize<16, UnsignedType>::type uint128;

typedef TypeOfSize<2, FloatType>::type float16; //Very unlikely to exist
typedef TypeOfSize<4, FloatType>::type float32;
typedef TypeOfSize<8, FloatType>::type float64;
typedef TypeOfSize<16, FloatType>::type float128; //Very unlikely to exist

#endif /* TYPES_H_ */
